# Generated by roxygen2: do not edit by hand

export(as_sparse_character)
export(as_sparse_double)
export(as_sparse_integer)
export(as_sparse_logical)
export(coerce_to_sparse_data_frame)
export(coerce_to_sparse_matrix)
export(coerce_to_sparse_tibble)
export(has_sparse_elements)
export(is_sparse_character)
export(is_sparse_double)
export(is_sparse_integer)
export(is_sparse_logical)
export(is_sparse_numeric)
export(is_sparse_vector)
export(sparse_addition_scalar)
export(sparse_character)
export(sparse_default)
export(sparse_division_scalar)
export(sparse_double)
export(sparse_dummy)
export(sparse_integer)
export(sparse_is_na)
export(sparse_lag)
export(sparse_logical)
export(sparse_mean)
export(sparse_median)
export(sparse_multiplication)
export(sparse_multiplication_scalar)
export(sparse_positions)
export(sparse_replace_na)
export(sparse_sd)
export(sparse_sqrt)
export(sparse_subtraction_scalar)
export(sparse_values)
export(sparse_var)
export(sparse_which_na)
export(sparsity)
import(rlang)
useDynLib(sparsevctrs, .registration = TRUE)
